MODULE chemMOD
	
	use rulesMOD, only:   &
	P_undef,              &
	P_dateLength,         &
	P_dirLength,          &
	P_wordLength,         &
	P_atmosAscvar,        &
	P_atmosAscName,       &
	P_debugMode,          &
	convert_specie_mech,  &
	P_OM_MaxTry,          &   
	P_OM_InitTime,        & 
	P_OM_InitAnl,         &
	P_OM_idName,          &
	zInterpOrder,         &
	P_unit
	
	USE utilsMOD, only:   &
	tArg,                 &
	sortZ,                &
	interpolationNN,      &
	interpolationBL,      &
	greg2jul,             &
	jul2greg,             &
	datec2datei,          &
	datei2datec
	
	USE chem1_list, only: &
	nspecies,             &
	spc_alloc,            &
	spc_name,             &
	fdda,                 &
	on,                   &
	weight,               &
	chemical_mechanism

	IMPLICIT NONE
	
	PRIVATE

	TYPE, PUBLIC :: tSpc
	
		REAL                        :: factor
		CHARACTER(LEN=P_wordLength) :: name,     &
					       chemName  
		INTEGER                     :: chemType, &
					       spcIdx
		TYPE(tSpc), POINTER         :: prior
		
	END TYPE	
		
	TYPE :: tChemList
	
		CHARACTER(LEN=P_wordLength) :: mech
		INTEGER                     :: nsp
		TYPE(tSpc), POINTER         :: spcs
	
	END TYPE tChemList
	
	TYPE :: tChem
				
		CHARACTER(LEN=P_dateLength)                            :: date,   &
									  chemMech
		INTEGER                                                :: nx,     &
			                                                  ny,     &
			                                                  nz,     &
			                                                  nv
		REAL                                                   :: lat(2), &
			                                                  lon(2), &
			                                                  xs,     &
			                                                  ys
		CHARACTER(LEN=P_wordLength), ALLOCATABLE, DIMENSION(:) :: varNames	   
		REAL, ALLOCATABLE, DIMENSION(:)                        :: levels
		REAL, ALLOCATABLE, DIMENSION(:,:,:,:)                  :: values
		
	END TYPE tChem	

	PUBLIC :: tChemList
	PUBLIC :: tChem
	PUBLIC :: config_chem
	PUBLIC :: config_chemList
	PUBLIC :: open_chem
	PUBLIC :: release_chem
	
	CONTAINS
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE config_chemList(chemList, ppbm,  chemType, ret)
	
		TYPE(tChemList), INTENT(OUT) :: chemList
		INTEGER, INTENT(IN)          :: chemType			
		LOGICAL, INTENT(IN)          :: ppbm		
		
		INTEGER                      :: i
		TYPE(tSpc), POINTER          :: tmpSpc
		CHARACTER(LEN=P_wordLength)  :: currSpcName
		REAL                         :: currFactor
		INTEGER, INTENT(OUT)         :: ret
		CHARACTER(LEN=P_wordLength)  :: msgOut(3)
			
			chemList%mech = chemical_mechanism
						
			NULLIFY(chemList%spcs)
			
			chemList%nsp = 0	
			
			DO i = 1, nspecies
				IF(spc_alloc(fdda,i) .GE. on)THEN
					
					chemList%nsp = chemList%nsp + 1					
					
					currSpcName = spc_name(i)
					currFactor  = weight(i)

					CALL convert_specie_mech(currSpcName, currFactor, chemical_mechanism, ppbm, ret)
					
					IF(ret .NE. 0)RETURN
										
					ALLOCATE(tmpSpc)
					
					tmpSpc%spcIdx   = chemList%nsp
					tmpSpc%name     = currSpcName
					tmpSpc%chemName = spc_name(i)
					tmpSpc%factor   = currFactor
					
					IF(spc_alloc(fdda,i) .NE. 1)THEN
						tmpSpc%chemType = spc_alloc(fdda,i)
					ELSE
						tmpSpc%chemType = chemType
					END IF
					
					tmpSpc%prior  => chemList%spcs
					chemList%spcs => tmpSpc
					
					IF(P_debugMode .GT. 1)THEN
					
						CALL message_output('--- DEBUGGING CHEMICAL LIST DEFINITION: ')
						WRITE(msgOut(1), '(F9.3)') weight(i)
						WRITE(msgOut(2), '(F9.3)') currFactor
						WRITE(msgOut(3), '(I2.2)') tmpSpc%chemType
						
						CALL message_output('CHEM1LIST - INPUT TYPE : '//TRIM(msgOut(3)))
						
						CALL message_output('CHEM1LIST - NAME       : '//TRIM(spc_name(i)))	
						CALL message_output('CHEM1LIST - WEIGTH     : '//TRIM(msgOut(1)))
						CALL message_output('DPCHEM    - NAME       : '//TRIM(currSpcName))	
						CALL message_output('DPCHEM    - WEIGTH     : '//TRIM(msgOut(2)))
						
					END IF
				END IF
			END DO
			
			ret = 0
	
	END SUBROUTINE config_chemList

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE config_chem(chemList, dpChem, nx, ny, nz, lat, lon, xs, ys, levels, ret)
	
		TYPE(tChemList), INTENT(IN)     :: chemList	
		TYPE(tChem), INTENT(OUT)        :: dpChem
		INTEGER, INTENT(IN)             :: nx, &
						   ny, &
						   nz
		REAL, DIMENSION(2), INTENT(IN)  :: lat, &
						   lon
		REAL, INTENT(IN)                :: xs, &
						   ys
		REAL, DIMENSION(nz), INTENT(IN) :: levels		
		INTEGER, INTENT(OUT)            :: ret

		CHARACTER(LEN=P_wordLength) :: msgOut(3)	
			
			
			
			ALLOCATE(dpChem%varNames(chemList%nsp))	   
			ALLOCATE(dpChem%levels(nz))
			ALLOCATE(dpChem%values(nx, ny, nz, chemList%nsp))
			
			dpChem%nx     = nx
			dpChem%ny     = ny
			dpChem%nz     = nz			
			dpChem%nv     = chemList%nsp			
			dpChem%lat    = lat
			dpChem%lon    = lon
			dpChem%xs     = xs
			dpChem%ys     = ys
			dpChem%levels = levels
			dpChem%chemMech = chemList%mech
			
			IF(P_debugMode .GT. 1)THEN
		
				CALL message_output('--- DEBUGGING CHEM - SETTINGS')
				
				WRITE(msgOut(1), '(I5.5)') nx
				WRITE(msgOut(2), '(I5.5)') ny
				WRITE(msgOut(3), '(I5.5)') nz
				
				CALL message_output('CHEM NX          : '//TRIM(msgOut(1)))
				CALL message_output('CHEM NY          : '//TRIM(msgOut(2)))
				CALL message_output('CHEM NZ          : '//TRIM(msgOut(3)))
				
				WRITE(msgOut(1), '(F9.3)') lat(1)
				WRITE(msgOut(2), '(F9.3)') lat(2)
				
				CALL message_output('CHEM INITIAL LAT : '//TRIM(msgOut(1)))
				CALL message_output('CHEM FINAL   LAT : '//TRIM(msgOut(2)))
				
				WRITE(msgOut(1), '(F9.3)') lon(1)
				WRITE(msgOut(2), '(F9.3)') lon(2)
				
				CALL message_output('CHEM INITIAL LON : '//TRIM(msgOut(1)))
				CALL message_output('CHEM FINAL   LON : '//TRIM(msgOut(2)))
				
			END IF
			
			ret = 0 

	END SUBROUTINE config_chem

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE open_chem(iArg, currDate, dpChem, chemList, ret)

		TYPE(tArg), INTENT(IN)          :: iArg
		CHARACTER(LEN=*), INTENT(IN)    :: currDate
		TYPE(tChem), INTENT(INOUT)      :: dpChem
		TYPE(tChemList), INTENT(IN)     :: chemList		
		INTEGER, INTENT(OUT)            :: ret

		LOGICAL                     :: fileok
		CHARACTER(LEN=P_dateLength) :: OMDate
		CHARACTER(LEN=P_dirLength)  :: file
		INTEGER                     :: i
		TYPE(tSpc), POINTER         :: tmpChemList		
			
			tmpChemList => chemList%spcs
			
			DO WHILE(ASSOCIATED(tmpChemList))
				SELECT CASE(tmpChemList%chemType)
				CASE(10) !MONTHLY SHADOZ PERFIL
					
					!NEEDS AN ASC FILE WITH 2 COLUMNS [LEVEL, VALUE] IN PPBV
					
					!IF WANT TO USE SPC NAME AS CHEM1LIST SWAP tmpChemList%name BY tmpChemList%chemName
					CALL create_filename_mshadoz(file, currDate, tmpChemList%name)
			
					INQUIRE(FILE=TRIM(iArg%chemInputDir)//TRIM(file), EXIST=fileok)
					
					IF(.NOT. fileok)THEN
						IF(P_debugMode .GT. 0) CALL message_output('CHEM FILE: '//TRIM(file)//' NOT FOUND!')
						ret = -1
						RETURN
					END IF
					
					CALL open_chem_mshadoz(TRIM(iArg%chemInputDir)//TRIM(file), currDate, tmpChemList%chemName, tmpChemList%factor, dpChem, tmpChemList%spcIdx, ret)
					
					IF(ret .NE. 0)THEN
						ret = -1
						RETURN
					END IF
					
				CASE(11) !BINARY CLIMATOLOGY
				
					!NEEDS INFO (METADATA) AND BINARYS
					CALL create_filename_climBin(file, currDate, tmpChemList%name)
					
					INQUIRE(FILE=TRIM(iArg%chemInputDir)//TRIM(file), EXIST=fileok)
					
					IF(.NOT. fileok)THEN
						IF(P_debugMode .GT. 0) CALL message_output('CHEM FILE: '//TRIM(file)//' NOT FOUND!')
						ret = -1
						RETURN
					END IF
					
					CALL open_chem_climBin(iArg%chemProj, TRIM(iArg%chemInputDir)//TRIM(file), currDate, tmpChemList%chemName, tmpChemList%factor, dpChem, tmpChemList%spcIdx, ret)
					
					IF(ret .NE. 0)THEN
						ret = -1
						RETURN
					END IF
					
				CASE(12) !OPERATIONAL MOCAGE (in PPBV)
					
					
					DO i = 1, P_OM_MaxTry
					
						CALL create_filename_operMocage(i, file, currDate, tmpChemList%name, OMDate)					
						INQUIRE(FILE=TRIM(iArg%chemInputDir)//TRIM(file), EXIST=fileok)
						IF(fileok)THEN
							EXIT
						ELSE
							IF(P_debugMode .GT. 0) CALL message_output('CHEM FILE: '//TRIM(file)//' NOT FOUND!')
						END IF					
					END DO
					
					IF(.NOT. fileok)THEN						
						ret = -1
						RETURN
					END IF					
#if USEGRIB==1	
					CALL open_chem_mocage(iArg%chemProj, TRIM(iArg%chemInputDir)//TRIM(file), OMDate, tmpChemList%chemName, tmpChemList%name, tmpChemList%factor, dpChem, tmpChemList%spcIdx, ret)
#else
					CALL message_output('______________________________________________________')
					CALL message_output('THESE EXECUTABLE WAS COMPILED WITHOUT GRIB API LIBRARY. PLEASE RE-COMPILE OR CHANGE YOUR CHEMICAL FILE INPUT TYPE.')
					CALL message_output('______________________________________________________')
					ret = -1
#endif
					
					IF(ret .NE. 0)THEN
						ret = -1
						RETURN
					END IF					
												
				CASE DEFAULT
				
					IF(P_debugMode .GT. 0) CALL message_output('CHEM FILE INPUT TYPE DOES NOT EXIST, CHECK YOUR CHEM1_LIST.')	
					ret = -1
					RETURN
				
				END SELECT
			
				tmpChemList => tmpChemList%prior
							
			END DO
	
			
	END SUBROUTINE open_chem

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE create_filename_mshadoz(file, currDate, spcName)
		
		CHARACTER(LEN=*), INTENT(OUT) :: file
		CHARACTER(LEN=*), INTENT(IN)  :: currDate, &
						 spcName
						 
		CHARACTER(LEN=3), DIMENSION(12) :: cMonths
		INTEGER                         :: mIndex
			
		DATA cMonths /'jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec'/
			
			!O3_shadoz_feb2004.txt
			
			READ(currDate(5:6),'(I2)')mIndex
			
			file = TRIM(spcName)//                                   &
				    '_shadoz_'//cMonths(mIndex)//currDate(1:4)// &
			    	    '.txt'
				
	END SUBROUTINE create_filename_mshadoz
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE create_filename_climBin(file, currDate, spcName)
		
		CHARACTER(LEN=*), INTENT(OUT) :: file
		CHARACTER(LEN=*), INTENT(IN)  :: currDate, &
						 spcName
						 
		CHARACTER(LEN=3), DIMENSION(12) :: cMonths
		INTEGER                         :: mIndex
                CHARACTER(LEN=2)                :: hIndex
			
		DATA cMonths /'jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec'/
			
			!climatology_NO_aug2008.info
			
			READ(currDate(5:6),'(I2)')mIndex
                        hIndex = currDate(9:10)	
		
			!file = 'climatology_'//TRIM(spcName)//'_'//cMonths(mIndex)//currDate(1:4)//'.info'
			file = 'climatology_'//cMonths(mIndex)//'_'//TRIM(spcName)//'_'//hIndex//'.info'	
			
	END SUBROUTINE create_filename_climBin
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE create_filename_operMocage(i, file, currDate, spcName, OMDate)

		INTEGER, INTENT(IN)           :: i
		CHARACTER(LEN=*), INTENT(OUT) :: file,     &
						 OMDate
		CHARACTER(LEN=*), INTENT(IN)  :: currDate, &
						 spcName	
						 
		CHARACTER(LEN=P_dateLength) :: tempDate
		INTEGER                     :: h, &
					       d, &
					       m, &
					       y
		REAL(KIND=8)                :: jD			
			
			CALL greg2jul(currDate,jD)
			CALL jul2greg((jD-i), tempDate)
			
			CALL datec2datei(h,d,m,y,tempDate)
			
			h = h + P_OM_InitAnl + ((i-1) * (P_OM_InitAnl + P_OM_InitTime))
			
			CALL datei2datec(h,d,m,y,tempDate)
			
			OMDate = tempDate
			
			file = 'Mocage_'//TRIM(spcName)//'_'//tempDate(1:8)//'120000_'//tempDate(9:10)			
	
	END SUBROUTINE create_filename_operMocage
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE open_chem_mshadoz(file, currDate, spcname, factor, dpChem, spcIdx, ret)
		
		CHARACTER(LEN=*), INTENT(IN)  :: file,     & 
						 currDate, &
						 spcname
		REAL, INTENT(IN)              :: factor
		TYPE(tChem), INTENT(INOUT)    :: dpChem
		INTEGER, INTENT(IN)           :: spcIdx
		INTEGER, INTENT(OUT) 	      :: ret
		
		TYPE tMShadoz
		
			REAL                    :: level, &
				                   value
			TYPE(tMShadoz), POINTER :: p
		
		END TYPE tMShadoz
			
		TYPE(tMshadoz), POINTER            :: tmpPShadoz, &
				                      PShadoz
		INTEGER                            :: fstat,      &
					              i,          &
						      z,          &
					              FShadozCt
		REAL                               :: tLevel, tValue
		INTEGER, DIMENSION(:), ALLOCATABLE :: tempZIndex
		REAL, DIMENSION(:,:), ALLOCATABLE  :: FShadoz
		CHARACTER(LEN=P_wordLength)        :: msgOut(3)
						
			FShadozCt = 0
									
			dpChem%varNames(spcIdx) = spcname
			
			IF(P_debugMode .GT. 0) CALL message_output('@@@>>>>>>>>>>>>>>>>>>>> OPENING MONTHLY SHADOZ PERFIL FILE : '//TRIM(file))
	
			NULLIFY(PShadoz)
					
			IF(P_debugMode .GT. 1) CALL message_output('--- DEBUGGING ORIGINAL SHADOZ PERFIL FOR : '//TRIM(spcName))
						
			!READ SHADOZ FILE AND SAVE THEIR INFORMATION IN A DYNAMIC ALLOCATABLE VARIABLE
			
			OPEN(UNIT=P_unit, FILE=TRIM(file), STATUS='OLD')
			DO
				READ(P_unit, *, IOSTAT=fstat)tLevel, tValue
				IF(fstat .NE. 0)EXIT
				
				FShadozCt = FShadozCt + 1
				
				ALLOCATE(tmpPShadoz)
				
				IF(P_debugMode .GT. 1)THEN
					
					WRITE(msgOut(1), '(F9.3)') tLevel
					WRITE(msgOut(2), '(F9.3)') tValue
										
					CALL message_output('Level : '//TRIM(msgOut(1))//' value: '//TRIM(msgOut(2)))
				
				END IF
				
				tmpPShadoz%level = tLevel
				tmpPShadoz%value = tValue
				
				tmpPShadoz%p => PShadoz
				PShadoz      => tmpPShadoz
			
			END DO
			
			CLOSE(P_unit)			
			
			!CONVERT A DYNAMIC ALLOCATABLE VARIABLE IN A NORMAL ALLOCATABLE VARIABLE
			
			ALLOCATE(FShadoz(2,FShadozCt))
			
			i = 1
			DO WHILE(ASSOCIATED(PShadoz))
			
				tmpPShadoz => PShadoz
				PShadoz    => tmpPShadoz%p
				
				Fshadoz(1,i) = tmpPShadoz%level
				Fshadoz(2,i) = tmpPShadoz%value

				DEALLOCATE(tmpPShadoz)
				i = i + 1
			
			END DO
			
			!IF zInterpOrder = 1 IT 
			
			ALLOCATE(tempZIndex(zInterpOrder))

			IF(zInterpOrder .GT. 1)THEN
				!ANOTHER KIND OF INTERPOLATION
				ret = -1
				RETURN
						
			ELSE
				!PERFORMS A NEAREST NEIGHBOOR INTERPOLATION
				
				IF(P_debugMode .GT. 1)THEN
					CALL message_output('> PERFORMING A VERTICAL NEAREST NEIGHBOOR INTERPOLATION FOR : '//TRIM(spcName))
					CALL message_output('--- DEBUGGING CHEM FOR : '//TRIM(spcName))
					
					WRITE(msgOut(1), '(I5.5)') dpChem%nx
					WRITE(msgOut(2), '(I5.5)') dpChem%ny
					WRITE(msgOut(3), '(I5.5)') dpChem%nz
				
					CALL message_output('NX          : '//TRIM(msgOut(1)))
					CALL message_output('NY          : '//TRIM(msgOut(2)))
					CALL message_output('NZ          : '//TRIM(msgOut(3)))
				
					WRITE(msgOut(1), '(F9.3)') dpChem%lat(1)
					WRITE(msgOut(2), '(F9.3)') dpChem%lat(2)
				
					CALL message_output('INITIAL LAT : '//TRIM(msgOut(1)))
					CALL message_output('FINAL   LAT : '//TRIM(msgOut(2)))
				
					WRITE(msgOut(1), '(F9.3)') dpChem%lon(1)
					WRITE(msgOut(2), '(F9.3)') dpChem%lon(2)
				
					CALL message_output('INITIAL LON : '//TRIM(msgOut(1)))
					CALL message_output('FINAL   LON : '//TRIM(msgOut(2)))
					
				END IF
				
				DO z = 1, dpChem%nz
					
					!RETURNS THE NEAREST SHADOZ LEVEL INDEX OF CURRENT CHEM LEVEL 
					CALL sortZ(FShadozCt, Fshadoz(1,:), dpChem%levels(z), zInterpOrder, tempZIndex)
					
					!FILLS CHEM VARIABLE WITH SHADOZ VALUE
					dpChem%values(:,:,z,spcIdx) = Fshadoz(2,tempZIndex(1)) * factor
					
					IF(P_debugMode .GT. 1)THEN						
						
						WRITE(msgOut(1), '(F9.3)') dpChem%levels(z)
						WRITE(msgOut(2), '(F9.3)') MINVAL(dpChem%values(:,:,z,spcIdx))	
						WRITE(msgOut(3), '(F9.3)') MAXVAL(dpChem%values(:,:,z,spcIdx))
						CALL message_output(TRIM(msgOut(1))//' min: '//TRIM(msgOut(2))//' max: '//TRIM(msgOut(3)) )
					END IF
						
				END DO
	
			END IF
			
			DEALLOCATE(tempZIndex)
			
			ret = 0			
			
	END SUBROUTINE open_chem_mshadoz	

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE open_chem_climBin(projType, file, currDate, spcname, factor, dpChem, spcIdx, ret)
		
		CHARACTER(LEN=*), INTENT(IN)  :: file,             &	
						 currDate,         &
						 spcname
		REAL, INTENT(IN)              :: factor
		TYPE(tChem), INTENT(INOUT)    :: dpChem
		INTEGER, INTENT(IN)           :: spcIdx,           &
						 projType
		INTEGER, INTENT(OUT) 	      :: ret
		
		LOGICAL                             :: fileok
		INTEGER                             :: lenBinName, &
					               i,          &
					               j,          &
						       z,          &
					               nx,         &
					               ny,         &
					               nz,         &
						       convUnit,   &
						       inByteSize	
		CHARACTER(LEN=P_dirLength)          :: binName
		REAL                                :: localUndef, &
					               xS,         &
					               yS	
		REAL, DIMENSION(2)                  :: lat,       &
						       lon
		INTEGER, DIMENSION(:), ALLOCATABLE  :: tempZIndex
		REAL, ALLOCATABLE, DIMENSION(:)     :: localLevels
		REAL, ALLOCATABLE, DIMENSION(:,:,:) :: localValues,     &
						       reproValues
		CHARACTER(LEN=P_wordLength)         :: msgOut(3)
		
			dpChem%varNames(spcIdx) = spcname
			
			IF(P_debugMode .GT. 0) CALL message_output('@@@>>>>>>>>>>>>>>>>>>>> OPENING CLIMATOLOGY FILE : '//TRIM(file))
			
			!CHECKING IF UNIT IS BUSY 
			!DO rUnit = 200, 300
       			!	INQUIRE(rUnit, OPENED=unitBusy)
       			!	IF(.NOT. unitBusy) EXIT
    			!END DO
			
			OPEN(UNIT=P_unit, FILE=TRIM(file), STATUS='OLD')
			
			!RETRIVE BINARY FILENAME
			READ(P_unit,*)binName
			
			lenBinName = LEN_TRIM(binName)
			
			i = INDEX(binName, '^')			
			binName=binName(i+1:lenBinName)
			
			i = INDEX(binName, '/')			
			j = INDEX(file, '/', .TRUE.)	
			
			IF(i .EQ. 0) binName = file(1:j)//TRIM(binName)
			
			IF(P_debugMode .GT. 1)CALL message_output('BINARY FILE : '//TRIM(binName))
	
			READ(P_unit,*)localUndef
			READ(P_unit,*)nx
			READ(P_unit,*)ny
			READ(P_unit,*)nz
			READ(P_unit,*)lon(1)
			READ(P_unit,*)lat(1)
			READ(P_unit,*)xS
			READ(P_unit,*)yS
						
			ALLOCATE(localLevels(nz))
			ALLOCATE(localValues(nx,ny,nz))			
			
			READ(P_unit,*)localLevels
			READ(P_unit,*)convUnit
			
			CLOSE(P_unit)			
			
			INQUIRE(FILE=TRIM(binName), EXIST=fileok)
			
			IF(.NOT. fileok)THEN
				IF(P_debugMode .GT. 0) CALL message_output('METADATA SEEMS OK BUT BINARY FILE : '//TRIM(file)//' NOT FOUND!')
				ret = -1
				RETURN
			END IF
			
			INQUIRE(IOLENGTH=inByteSize)localValues	
			
			OPEN (UNIT = P_unit, FILE = TRIM(binName), ACCESS = 'DIRECT', &
			       RECL = inByteSize, STATUS = 'OLD')
			READ(P_unit, REC=1)localValues
			CLOSE(P_unit)			
						
			IF(P_debugMode .GT. 1)THEN
		
				CALL message_output('--- DEBUGGING ORIGINAL CLIMATOLOGY FOR : '//TRIM(spcName))
				
				WRITE(msgOut(1), '(I5.5)') nx
				WRITE(msgOut(2), '(I5.5)') ny
				WRITE(msgOut(3), '(I5.5)') nz
				
				CALL message_output('NX          : '//TRIM(msgOut(1)))
				CALL message_output('NY          : '//TRIM(msgOut(2)))
				CALL message_output('NZ          : '//TRIM(msgOut(3)))
				
				WRITE(msgOut(1), '(F9.3)') lat(1)
				WRITE(msgOut(2), '(F9.3)') lat(2)
				
				CALL message_output('INITIAL LAT : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LAT : '//TRIM(msgOut(2)))
				
				WRITE(msgOut(1), '(F9.3)') lon(1)
				WRITE(msgOut(2), '(F9.3)') lon(2)
				
				CALL message_output('INITIAL LON : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LON : '//TRIM(msgOut(2)))
								
				DO j = 1, nz
					WRITE(msgOut(1), '(F9.3)') localLevels(j)						
					WRITE(msgOut(2), '(F9.3)') MINVAL(localValues(:,:,j))	
					WRITE(msgOut(3), '(F9.3)') MAXVAL(localValues(:,:,j))
					CALL message_output(TRIM(msgOut(1))//' min: '//TRIM(msgOut(2))//' max: '//TRIM(msgOut(3)) )
				END DO
				
			END IF			

			lat(2) = lat(1) + (ys*ny)
			lon(2) = lon(1) + (xs*nx)
			
			!REPROJECTS CLIMATOLOGY DATA TO THE SAME DIMENSION OF CHEM
			
			ALLOCATE(reproValues(dpChem%nx, dpChem%ny, nz))
			
			reproValues = P_undef
			
			SELECT CASE(projType)
			CASE(0) !NN INTERPOLATION
				IF(P_debugMode .GT. 1) CALL message_output('> PERFORMING A HORIZONTAL NEAREST NEIGHBOOR INTERPOLATION FOR : '//TRIM(spcName))
				
				CALL interpolationNN(localValues, nx, ny, nz, lat, lon, xs, ys, &
				                     reproValues, dpChem%nx, dpChem%ny, nz, dpChem%lat, dpChem%lon, dpChem%xs, dpChem%ys)
			CASE(1) !BILINEAR INTERPOLATION
				IF(P_debugMode .GT. 1) CALL message_output('> PERFORMING A HORIZONTAL BILINEAR INTERPOLATION FOR : '//TRIM(spcName))
				
				CALL interpolationBL(localValues, nx, ny, nz, lat, lon, xs, ys, &
				                     reproValues, dpChem%nx, dpChem%ny, nz, dpChem%lat, dpChem%lon, dpChem%xs, dpChem%ys)
			CASE DEFAULT
				IF(P_debugMode .GT. 0) CALL message_output('INTERPOLATION METHOD NOT ALLOWED/DEFINED')				
				ret = -1
				RETURN
			END SELECT
			
			DEALLOCATE(localValues)
			
			!CHECK IF IS ALREADY IN PPBM
			IF(convUnit .EQ. 0)THEN
			WHERE(reproValues .NE. P_undef) reproValues = reproValues * factor
			!	DO z = 1, nz
			!		DO j = 1, dpChem%ny
			!			DO i = 1, dpChem%nx				
			!				IF(reproValues(i,j,z) .NE. P_undef) reproValues(i,j,z) = reproValues(i,j,z) * factor
			!			END DO
			!		END DO
			!	END DO
			END IF
			
			ALLOCATE(tempZIndex(zInterpOrder))
			
			IF(zInterpOrder .GT. 1)THEN
				!ANOTHER KIND OF INTERPOLATION
				ret = -1
				RETURN
						
			ELSE
				!PERFORMS A NEAREST NEIGHBOOR INTERPOLATION
				
				IF(P_debugMode .GT. 1) CALL message_output('> PERFORMING A VERTICAL NEAREST NEIGHBOOR INTERPOLATION FOR : '//TRIM(spcName))
				
				DO z = 1, dpChem%nz
					
					!RETURNS THE NEAREST SHADOZ LEVEL INDEX OF CURRENT CHEM LEVEL 
					CALL sortZ(nz, localLevels, dpChem%levels(z), zInterpOrder, tempZIndex)
					
					!FILLS CHEM VARIABLE WITH CLIMATOLOGY VALUE
					dpChem%values(:,:,z,spcIdx) = reproValues(:,:,tempZIndex(1))					
				END DO
	
			END IF							
				
			IF(P_debugMode .GT. 1)THEN
		
				CALL message_output('--- DEBUGGING CHEM FOR : '//TRIM(spcName))
				
				WRITE(msgOut(1), '(I5.5)') dpChem%nx
				WRITE(msgOut(2), '(I5.5)') dpChem%ny
				WRITE(msgOut(3), '(I5.5)') dpChem%nz
				
				CALL message_output('NX          : '//TRIM(msgOut(1)))
				CALL message_output('NY          : '//TRIM(msgOut(2)))
				CALL message_output('NZ          : '//TRIM(msgOut(3)))
				
				WRITE(msgOut(1), '(F9.3)') dpChem%lat(1)
				WRITE(msgOut(2), '(F9.3)') dpChem%lat(2)
				
				CALL message_output('INITIAL LAT : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LAT : '//TRIM(msgOut(2)))
				
				WRITE(msgOut(1), '(F9.3)') dpChem%lon(1)
				WRITE(msgOut(2), '(F9.3)') dpChem%lon(2)
				
				CALL message_output('INITIAL LON : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LON : '//TRIM(msgOut(2)))
				
				DO j = 1, dpChem%nz
					WRITE(msgOut(1), '(F9.3)') dpChem%levels(j)						
					WRITE(msgOut(2), '(F9.3)') MINVAL(dpChem%values(:,:,j,spcIdx), MASK=dpChem%values(:,:,j,spcIdx) .NE. P_undef)	
					WRITE(msgOut(3), '(F9.3)') MAXVAL(dpChem%values(:,:,j,spcIdx), MASK=dpChem%values(:,:,j,spcIdx) .NE. P_undef)
					CALL message_output(TRIM(msgOut(1))//' min: '//TRIM(msgOut(2))//' max: '//TRIM(msgOut(3)) )
				END DO
				
			END IF			
			
			DEALLOCATE(localLevels)			
			DEALLOCATE(reproValues)
			DEALLOCATE(tempZIndex)
			ret = 0	
	
	END SUBROUTINE open_chem_climBin
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

#if USEGRIB==1	
	
	SUBROUTINE open_chem_mocage(projType, file, currDate, spcName, MOSpcName, factor, dpChem, spcIdx, ret)

		USE grib_api

		CHARACTER(LEN=*), INTENT(IN)  :: file,             &	
						 currDate,         &
						 spcName,          &
						 MOSpcName
		REAL, INTENT(IN)              :: factor
		TYPE(tChem), INTENT(INOUT)    :: dpChem
		INTEGER, INTENT(IN)           :: spcIdx,           &
						 projType
		INTEGER, INTENT(OUT) 	      :: ret
		
		
		TYPE Tmocage
 
 			INTEGER                        	  :: nx, &
	       		                             	     ny, &
		                                	     z
			REAL                              :: lat(2), &
						             lon(2), &
						             xs,     &
						             ys					      
 			REAL, DIMENSION(:,:), ALLOCATABLE :: value	
			TYPE(Tmocage), POINTER            :: p
 
 		END TYPE Tmocage
	
		TYPE(Tmocage), POINTER              :: dataMocage, &
		                                       tmpMocage		
		CHARACTER(LEN=P_wordLength)         :: gDate,       &
					               gTime,       &
					               gTypeGrid
		INTEGER                             :: gribFile,    &
                	                               gribId,      &
	              	                               gRet,        &
					               gribVarCode, &
					               centerId,    &
					               zUnitId,     &
					               gLevel,      &
						       gNPts(4),    &
						       i,           &
						       j,           &
						       z,           &
						       localNX,     &
						       localNY,     &
						       localNZ
		REAL                                :: localUndef,  &
						       localLat(2), &
						       localLon(2), &
						       localXS,     &
						       localYS,     &
						       tmp
		CHARACTER(LEN=P_wordLength)         :: msgOut(3)
		INTEGER, DIMENSION(:), ALLOCATABLE  :: tempZIndex
		REAL, ALLOCATABLE, DIMENSION(:)     :: tmpGData,    &
						       localLevel
		REAL, ALLOCATABLE, DIMENSION(:,:,:) :: localValues, &
						       reproValues	
			
			localNZ = 0
			
			ret     = -1
			
			dpChem%varNames(spcIdx) = spcname
			
			IF(P_debugMode .GT. 1)CALL message_output('MOCAGE FILE : '//TRIM(file))
	
			NULLIFY(dataMocage)
		
			CALL grib_open_file(gribFile, TRIM(file),'r') 
			
			CALL grib_new_from_file(gribFile, gribId, gRet)
			
			DO WHILE( gRet .NE. GRIB_END_OF_FILE )	
			
				CALL grib_get_string(gribId, 'dataDate', gDate)
				
				CALL grib_get_string(gribId, 'startStepInHours', gTime)	
				
				IF(TRIM(currDate) .NE. (TRIM(gDate)//TRIM(gTime)))THEN
					
					IF(P_debugMode .GT. 1)THEN					
						CALL message_output('Wrong date in your mocage file.')
						CALL message_output('expected date: '//TRIM(currDate))
						CALL message_output('file date: '//TRIM(gDate)//TRIM(gTime))					
					END IF
									
					CALL grib_release(gribId)
					CALL grib_new_from_file(gribFile, gribId, gRet)
					
					CYCLE
				
				END IF
				
				ret = 1
				
				CALL grib_get_string(gribId, 'typeOfGrid', gTypeGrid)
				IF(TRIM(gTypeGrid) .NE. 'regular_ll')THEN
					
					IF(P_debugMode .GT. 1)THEN
						CALL message_output('Wrong grid type: '//TRIM(gTypeGrid))
						CALL message_output('expected: regular_ll')
						CALL message_output('check your grib file or modify chemMOD.f90')
					END IF
					
					ret = -1
					RETURN
				END IF
		
				CALL grib_get(gribId, 'indicatorOfParameter', gribVarCode)
				IF(TRIM(P_OM_idName(gribVarCode)) .NE. TRIM(MOSpcName))THEN
					
					IF(P_debugMode .GT. 1)THEN
						WRITE(msgOut(1), '(I3)') gribVarCode
						CALL message_output('Wrong variable ID: '//TRIM(msgOut(1)))
						CALL message_output(TRIM(spcname)//' <-> '//TRIM(P_OM_idName(gribVarCode)))
						CALL message_output('check your grib file and your parameters definition or modify chemMOD.f90/rulesMOD.f90')
					END IF
					
					ret = -1
					RETURN
				END IF
	
				CALL grib_get(gribId, 'identificationOfOriginatingGeneratingCentre', centerId)
				IF(centerId .NE. 85)THEN
				
					IF(P_debugMode .GT. 1)THEN
						WRITE(msgOut(1), '(I3)') centerId				
						CALL message_output('your data isnt from [ French Weather Service - Toulouse ]')
						CALL message_output('your generating center is :'//TRIM(msgOut(1)))
						CALL message_output('check at: http://www.ecmwf.int/publications/manuals/libraries/gribex/wmoCodeTable0.html')
					END IF
				
					ret = -1
					RETURN
				END IF
		
				CALL grib_get(gribId, 'indicatorOfTypeOfLevel', zUnitId)
				IF(zUnitId .NE. 100)THEN
				
					IF(P_debugMode .GT. 1)THEN
						WRITE(msgOut(1), '(I3)') zUnitId
						CALL message_output('Error: Wrong level pressure.')
						CALL message_output('Expected (hPa)')
						CALL message_output('Your indicator = '//msgOut(1))
						CALL message_output('check at: http://www.ecmwf.int/publications/manuals/libraries/gribex/wmoCodeTable3.html')
					END IF
		
					ret = -1
					RETURN
				END IF
				
				localNZ = localNZ + 1
				
				ALLOCATE(tmpMocage)
								
				CALL grib_get(gribId, 'level', tmpMocage%z)
				CALL grib_get(gribId, 'numberOfPoints', gNPts)
		
				CALL grib_get(gribId, 'numberOfPointsAlongAParallel', tmpMocage%nx)		
				CALL grib_get(gribId, 'numberOfPointsAlongAMeridian', tmpMocage%ny)
		
				CALL grib_get(gribId, 'latitudeOfFirstGridPointInDegrees', tmpMocage%lat(1))
				CALL grib_get(gribId, 'longitudeOfFirstGridPointInDegrees', tmpMocage%lon(1))
				CALL grib_get(gribId, 'latitudeOfLastGridPointInDegrees', tmpMocage%lat(2))
				CALL grib_get(gribId, 'longitudeOfLastGridPointInDegrees', tmpMocage%lon(2))
				
				CALL grib_get(gribId, 'jDirectionIncrementInDegrees', tmpMocage%ys)
  				CALL grib_get(gribId, 'iDirectionIncrementInDegrees', tmpMocage%xs)
		
				CALL grib_get(gribId, 'missingValue', localUndef)
				
				ALLOCATE(tmpGData(gNPts(1)))
				
				CALL grib_get(gribId, 'values', tmpGData)
				
				ALLOCATE(tmpMocage%value(tmpMocage%nx, tmpMocage%ny))
				
				DO j = 1, tmpMocage%ny
					DO i = 1, tmpMocage%nx
						tmpMocage%value(i,j) = tmpGData( (j-1)*(tmpMocage%nx)+i ) 
					END DO
				END DO		
		
				tmpMocage%p => dataMocage
				dataMocage  => tmpMocage
				
				DEALLOCATE(tmpGData)
				
				CALL grib_release(gribId)
				CALL grib_new_from_file(gribFile, gribId, gRet)		
			END DO
			
			CALL GRIB_CLOSE_FILE(gribFile)
			
			localNX  = dataMocage%nx
			localNY  = dataMocage%ny
			localLat = dataMocage%lat
			localLon = dataMocage%lon
			localXS  = dataMocage%xs
			localYS  = dataMocage%ys 
			
			ALLOCATE(localValues(localNX, localNY, localNZ))
			ALLOCATE(localLevel(localNZ))
			
			i = 1
			DO WHILE(ASSOCIATED(dataMocage))
			
				tmpMocage  => dataMocage
				dataMocage => tmpMocage%p
				
				localLevel(i)      = tmpMocage%z
				localValues(:,:,i) = tmpMocage%value	
								
				DEALLOCATE(tmpMocage%value)
				DEALLOCATE(tmpMocage)
				i = i + 1
			
			END DO

			IF(P_debugMode .GT. 1)THEN
		
				CALL message_output('--- DEBUGGING ORIGINAL OPERATIONAL MOCAGE FOR : '//TRIM(spcName))
				
				WRITE(msgOut(1), '(I5.5)') localNX
				WRITE(msgOut(2), '(I5.5)') localNY
				WRITE(msgOut(3), '(I5.5)') localNZ
				
				CALL message_output('NX          : '//TRIM(msgOut(1)))
				CALL message_output('NY          : '//TRIM(msgOut(2)))
				CALL message_output('NZ          : '//TRIM(msgOut(3)))
				
				WRITE(msgOut(1), '(F9.3)') localLat(1)
				WRITE(msgOut(2), '(F9.3)') localLat(2)
				
				CALL message_output('INITIAL LAT : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LAT : '//TRIM(msgOut(2)))
				
				WRITE(msgOut(1), '(F9.3)') localLon(1)
				WRITE(msgOut(2), '(F9.3)') localLon(2)
				
				CALL message_output('INITIAL LON : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LON : '//TRIM(msgOut(2)))
								
				DO j = 1, localNZ
					WRITE(msgOut(1), '(F9.3)') localLevel(j)						
					WRITE(msgOut(2), '(F9.3)') MINVAL(localValues(:,:,j))	
					WRITE(msgOut(3), '(F9.3)') MAXVAL(localValues(:,:,j))
					CALL message_output(TRIM(msgOut(1))//' min: '//TRIM(msgOut(2))//' max: '//TRIM(msgOut(3)) )
				END DO
				
			END IF
			
			!REPROJECTS CLIMATOLOGY DATA TO THE SAME DIMENSION OF CHEM
			
			ALLOCATE(reproValues(dpChem%nx, dpChem%ny, localNZ))
			
			reproValues = P_undef
			
			SELECT CASE(projType)
			CASE(0) !NN INTERPOLATION
				IF(P_debugMode .GT. 1) CALL message_output('> PERFORMING A HORIZONTAL NEAREST NEIGHBOOR INTERPOLATION FOR : '//TRIM(spcName))
				
				CALL interpolationNN(localValues, localNX, localNY, localNZ, localLat, localLon, localXS, localYS, &
				                     reproValues, dpChem%nx, dpChem%ny, localNZ, dpChem%lat, dpChem%lon, dpChem%xs, dpChem%ys)
			CASE(1) !BILINEAR INTERPOLATION
				IF(P_debugMode .GT. 1) CALL message_output('> PERFORMING A HORIZONTAL BILINEAR INTERPOLATION FOR : '//TRIM(spcName))
				
				CALL interpolationBL(localValues, localNX, localNY, localNZ, localLat, localLon, localXS, localYS, &
				                     reproValues, dpChem%nx, dpChem%ny, localNZ, dpChem%lat, dpChem%lon, dpChem%xs, dpChem%ys)
			CASE DEFAULT
				IF(P_debugMode .GT. 0) CALL message_output('INTERPOLATION METHOD NOT ALLOWED/DEFINED')				
				ret = -1
				RETURN
			END SELECT
			
			DEALLOCATE(localValues)
			
			!CONVERT TO PPBM
			WHERE(reproValues .NE. P_undef) reproValues = reproValues * factor
			
			!DO z = 1, localNZ
			!	DO j = 1, dpChem%ny
			!		DO i = 1, dpChem%nx				
			!			IF(reproValues(i,j,z) .NE. P_undef) reproValues(i,j,z) = reproValues(i,j,z) * factor
			!		END DO
			!	END DO
			!END DO
			
			ALLOCATE(tempZIndex(zInterpOrder))
			
			IF(zInterpOrder .GT. 1)THEN
				!ANOTHER KIND OF INTERPOLATION
				ret = -1
				RETURN
						
			ELSE
				!PERFORMS A NEAREST NEIGHBOOR INTERPOLATION
				
				IF(P_debugMode .GT. 1) CALL message_output('> PERFORMING A VERTICAL NEAREST NEIGHBOOR INTERPOLATION FOR : '//TRIM(spcName))
				
				DO z = 1, dpChem%nz
					
					!RETURNS THE NEAREST SHADOZ LEVEL INDEX OF CURRENT CHEM LEVEL 
					CALL sortZ(localNZ, localLevel, dpChem%levels(z), zInterpOrder, tempZIndex)
					
					
					IF(localNZ .EQ. 15)THEN
					
						IF( dpChem%levels(z) .GT. 50 .AND. dpChem%levels(z) .LT. 100 )THEN
					
							!PRINT*, 'EXP', localLevel(tempZIndex(1)), dpChem%levels(z)
							!IF(TRIM(spcName) .EQ. 'CO')THEN
							!	dpChem%values(:,:,z,spcIdx) = 20
							!ELSE
								
								!tmp                       = log( reproValues(1,1,tempZIndex(1)) * 0.001 )
								!(-(LOG(reproValues(:,:,tempZIndex(1))*0.001))/50.)
								dpChem%values(:,:,z,spcIdx) = ( reproValues(:,:,tempZIndex(1)) * exp(-0.046 * (100 - dpChem%levels(z)) ) )	
							!END IF							
						
						ELSE IF( dpChem%levels(z) .LE. 50 )THEN
					
							!PRINT*, 'EXP DO 50', localLevel(tempZIndex(1)), dpChem%levels(z)	
							IF(TRIM(spcName) .EQ. 'CO')THEN 
								dpChem%values(:,:,z,spcIdx) = 20
							ELSE
								!tmp                       = (-(LOG(reproValues(:,:,tempZIndex(1))*0.001))/50.)
								dpChem%values(:,:,z,spcIdx) = ( reproValues(:,:,tempZIndex(1)) * exp(-0.046 * (50) ) )
							END IF				
					
						ELSE
							dpChem%values(:,:,z,spcIdx) = reproValues(:,:,tempZIndex(1))						
						END IF
					
					ELSE
					
						!FILLS CHEM VARIABLE WITH CLIMATOLOGY VALUE					
						!PRINT*, 'NORMAL', localLevel(tempZIndex(1)), dpChem%levels(z)
						dpChem%values(:,:,z,spcIdx) = reproValues(:,:,tempZIndex(1))	
					
					
					END IF 
					!PRINT*, dpChem%levels(z), MAXVAL(dpChem%values(:,:,z,spcIdx)), localLevel(tempZIndex(1))
					  
				
				END DO
			END IF							
				
			IF(P_debugMode .GT. 1)THEN
		
				CALL message_output('--- DEBUGGING CHEM FOR : '//TRIM(spcName))
				
				WRITE(msgOut(1), '(I5.5)') dpChem%nx
				WRITE(msgOut(2), '(I5.5)') dpChem%ny
				WRITE(msgOut(3), '(I5.5)') dpChem%nz
				
				CALL message_output('NX          : '//TRIM(msgOut(1)))
				CALL message_output('NY          : '//TRIM(msgOut(2)))
				CALL message_output('NZ          : '//TRIM(msgOut(3)))
				
				WRITE(msgOut(1), '(F9.3)') dpChem%lat(1)
				WRITE(msgOut(2), '(F9.3)') dpChem%lat(2)
				
				CALL message_output('INITIAL LAT : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LAT : '//TRIM(msgOut(2)))
				
				WRITE(msgOut(1), '(F9.3)') dpChem%lon(1)
				WRITE(msgOut(2), '(F9.3)') dpChem%lon(2)
				
				CALL message_output('INITIAL LON : '//TRIM(msgOut(1)))
				CALL message_output('FINAL   LON : '//TRIM(msgOut(2)))
				
				DO j = 1, dpChem%nz
					WRITE(msgOut(1), '(F12.3)') dpChem%levels(j)						
					WRITE(msgOut(2), '(F12.3)') MINVAL(dpChem%values(:,:,j,spcIdx), MASK=dpChem%values(:,:,j,spcIdx) .NE. P_undef)	
					WRITE(msgOut(3), '(F12.3)') MAXVAL(dpChem%values(:,:,j,spcIdx), MASK=dpChem%values(:,:,j,spcIdx) .NE. P_undef)
					CALL message_output(TRIM(msgOut(1))//' min: '//TRIM(msgOut(2))//' max: '//TRIM(msgOut(3)) )
				END DO
				
			END IF			
			
			DEALLOCATE(localLevel)			
			DEALLOCATE(reproValues)
			DEALLOCATE(tempZIndex)
			
			ret = 0	
				
	END SUBROUTINE open_chem_mocage
#endif
		
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE release_chem(chemList, dpChem, clearAll)
	
		TYPE(tChemList), INTENT(INOUT) :: chemList
		TYPE(tChem), INTENT(INOUT)     :: dpChem
		LOGICAL, INTENT(IN)            :: clearAll
		
		TYPE(tSpc), POINTER :: tmpChemList

			
			IF(clearAll)THEN
			
				DO WHILE(ASSOCIATED(chemList%spcs))
			
					tmpChemList=>chemList%spcs
					chemList%spcs=>tmpChemList%prior
	
					DEALLOCATE(tmpChemList)			
				END DO
				
			ELSE

				IF(ALLOCATED(dpChem%varNames))DEALLOCATE(dpChem%varNames)
				IF(ALLOCATED(dpChem%levels))DEALLOCATE(dpChem%levels)
				IF(ALLOCATED(dpChem%values))DEALLOCATE(dpChem%values)
			END IF


	END SUBROUTINE release_chem

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

END MODULE chemMOD
