
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

PROGRAM main

#if USEPARALELL==1	
	USE mpiMOD, only:   &
	create_mpi,         &
	destroy_mpi,        &
	bCastT_mpi,         &
	messageC_mpi,       &
	time_mpi
#endif	

	USE rulesMOD, only: &
	P_dateLength,       &  
	P_debugMode ,       &
	P_unit
	
	USE utilsMOD, only: &
	tArg,               &
	step2date,          &
	load_arg
		
	
	USE atmosMOD, only: &
	tAtmos,             &
	open_atmos,         &
	release_atmos
	
	USE chemMOD, only:  &
	tChemList,          &
	tChem,              &
	config_chem,        &
	config_chemList,    &
	open_chem,          &
	release_chem
			
	USE icMOD, only:    &
	write_debug_bin,    &
	ic_out
	
	IMPLICIT NONE
	
	!MPI
	INTEGER :: pId,   &
		   nProc, &
		   typeID
		   
	!BENCHMARK
	DOUBLE PRECISION      :: time1, time2
	CHARACTER(LEN=50)     :: msgOut(1)
	!AUX
	LOGICAL               :: unitBusy
	INTEGER               ::  i, &
				  ret
				       
	CHARACTER(LEN=P_dateLength) :: currDate
	
	!DATA STRUCT VARIABLES
	TYPE(tArg)   :: iArg, outArg
	TYPE(tAtmos) :: dpAtmos
	TYPE(tChem)  :: dpChem
	
	!CHEM
	TYPE(tChemList) chemList	
	
		
		!ENVIORNMENT INITIALIZATION						
		pId       = 0 
		nProc     = 1 		
		
#if USEPARALELL==1

		CALL create_mpi(pId, nProc)			
#endif
		
		!LOADING INPUT LIST	
		IF(pId .EQ. 0)THEN
			CALL load_arg('', iArg, ret)
			IF(ret .NE. 0)THEN
				CALL message_output('ERROR: INPUT LIST NOT FOUND!')
#if USEPARALELL==1
				CALL destroy_mpi()
#endif
				STOP			
			END IF
		END IF		

#if USEPARALELL==1

		!SHARE INPUT ARGUMENTS		
		CALL bCastT_mpi(iArg, 0)
		CALL time_mpi(time1) 	
	
		IF(iArg%steps(2) .LT. nProc)THEN
			CALL message_output('ERROR: NUMBER OF PROCESSORS GREATER THAN TIMESTEP VALUE IN INPUTLIST')
			CALL destroy_mpi()
			STOP		
		END IF
#endif
		
		!CHECKING IF UNIT IS BUSY 
		DO P_unit = 200, 300
       			INQUIRE(P_unit, OPENED=unitBusy)
       			IF(.NOT. unitBusy) EXIT
    		END DO
		
		!RETRIEVE INFO FROM CHEM1_LIST FILE
		IF(iArg%chemMerge)THEN
				
			CALL config_chemList(chemList, iArg%ppbmConvert, iArg%chemType, ret)
				
			IF(ret .NE. 0)THEN
				CALL message_output('ERROR: WRONG CHEMLIST CONFIGURATION')
#if USEPARALELL==1			
				CALL destroy_mpi()
#endif
				STOP
			END IF
		END IF		
		
		!EACH PROCESSOR GETS ONE TIMESTEP
	
		DO i = pId, iArg%steps(2)-1, nProc	
		
			!CONVERTING STEP TO GREGORIAN DATE
			CALL step2date(iArg%dateChar, currDate, i, iArg%steps(1))
						
			!CALL time_mpi(time2)		
			!WRITE(msgOut(1), '(F9.3)') (time2-time1)
			!CALL message_output('___________________________________Antes Atmos: '//TRIM(msgOut(1)))
			
			!LOAD ATMOS VARIABLE
			CALL open_atmos(iArg, currDate, dpAtmos, ret)
			
			IF(ret .NE. 0)THEN				
				
				CALL message_output('@@@>>>>>>>>>>>>>>>>>>>> ATMOS DATA STEP : '//currDate//' ERROR')
				CALL release_atmos(dpAtmos)
				CYCLE
			END IF
			
			IF(P_debugMode .GT. 0) CALL message_output('@@@>>>>>>>>>>>>>>>>>>>> ATMOS DATA STEP : '//currDate//' DONE')
			
			!CALL time_mpi(time2)		
			!WRITE(msgOut(1), '(F9.3)') (time2-time1)
			!CALL message_output('___________________________________Antes Chem: '//TRIM(msgOut(1)))
			
			IF(iArg%chemMerge)THEN
				
				!SET UP CHEM VARIABLE USING ATMOS DIMENSION AS INPUT PARAMETER
				CALL config_chem(chemList, dpChem, dpAtmos%nx, dpAtmos%ny, dpAtmos%nz, dpAtmos%lat, dpAtmos%lon, dpAtmos%xs, dpAtmos%ys, dpAtmos%levels, ret)
				
				IF(ret .NE. 0)THEN
					IF(P_debugMode .GT. 0) CALL message_output('@@@>>>>>>>>>>>>>>>>>>>> CHEM DATA STEP : '//currDate//' ERROR')
					CALL release_atmos(dpAtmos)
					CALL release_chem(chemList, dpChem, .FALSE.)
					CYCLE
				END IF				

				CALL open_chem(iArg, currDate, dpChem, chemList, ret)
				
				IF(ret .NE. 0)THEN
					IF(P_debugMode .GT. 0) CALL message_output('@@@>>>>>>>>>>>>>>>>>>>> CHEM DATA STEP : '//currDate//' ERROR')
					CALL release_atmos(dpAtmos)
					CALL release_chem(chemList, dpChem, .FALSE.)
					CYCLE
				END IF
			
				IF(P_debugMode .GT. 0) CALL message_output('@@@>>>>>>>>>>>>>>>>>>>> CHEM DATA STEP : '//currDate//' DONE')
			
			END IF
			
			!CALL time_mpi(time2)		
			!WRITE(msgOut(1), '(F9.3)') (time2-time1)
			!CALL message_output('___________________________________Antes Out debug: '//TRIM(msgOut(1)))
									
			IF(P_debugMode .GT. 2) CALL write_debug_bin(dpAtmos, dpChem, iArg%chemMerge, iArg%outDir)
			
			!CALL time_mpi(time2)		
			!WRITE(msgOut(1), '(F9.3)') (time2-time1)
			!CALL message_output('___________________________________Antes Out final: '//TRIM(msgOut(1)))
			CALL ic_out(dpAtmos, dpChem, iArg%chemMerge, iArg%outDir, iArg%outPrefix, iArg%outSufix, iArg%outType)
			
			IF(P_debugMode .GT. 0) CALL message_output('@@@>>>>>>>>>>>>>>>>>>>> OUTPUT FOR : '//currDate//' DONE')
			
			!RELEASE ALL
			CALL release_atmos(dpAtmos)
			IF(iArg%chemMerge) CALL release_chem(chemList, dpChem, .FALSE.)
		
		END DO	
		
		IF(iArg%chemMerge) CALL release_chem(chemList, dpChem, .TRUE.)

#if USEPARALELL==1
		CALL time_mpi(time2)		
		WRITE(msgOut(1), '(F9.3)') (time2-time1)
		CALL message_output('__________________X________________FINAL TIME: '//TRIM(msgOut(1))//'secs')		 
		CALL destroy_mpi()
#endif


END PROGRAM main

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

SUBROUTINE message_output(msg)

#if USEPARALELL==1	
	USE mpiMOD, only:   &
	messageC_mpi
#endif	
	
	CHARACTER(LEN=*), INTENT(IN) :: msg
		
#if USEPARALELL==1
		CALL messageC_mpi(TRIM(msg))
#else
		WRITE(*,*) TRIM(msg)
#endif
		
END SUBROUTINE message_output
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

