MODULE rulesMOD
	
	IMPLICIT NONE
	
	PUBLIC
	
	
		!MOVE TO PARAMETER MODULE
	INTEGER, PARAMETER :: P_dateLength  = 10
	INTEGER, PARAMETER :: P_dirLength   = 100
	INTEGER, PARAMETER :: P_wordLength  = 20
	
	INTEGER, PARAMETER :: P_debugMode   = 1

	LOGICAL, PARAMETER :: P_debugMpi = .TRUE.
        INTEGER, PARAMETER :: P_stdOut   = 1!6	
	 
	INTEGER, PARAMETER :: zInterpOrder = 1
	
	REAL, PARAMETER    :: P_undef       = -9999.999
	
	REAL, PARAMETER    :: P_airWeight   = 28.96 


	!>>> ATMOS INPUT RELATED <<<

	INTEGER, PARAMETER                                    :: P_atmosAscvar = 5	
	CHARACTER(LEN=P_wordLength), DIMENSION(P_atmosAscvar) :: P_atmosAscName	
	DATA P_atmosAscName /'U', 'V', 'TEMPK', 'GEO', 'UR' /

	!>>> END <<<
	
	!>>> OPERATIONAL MOCAGE RELATED <<<
	
	CHARACTER(10), DIMENSION(30) :: P_OM_idName
	INTEGER, PARAMETER 	     :: P_OM_MaxTry   = 3
	INTEGER, PARAMETER 	     :: P_OM_InitTime = 12
	INTEGER, PARAMETER 	     :: P_OM_InitAnl  = 12	 
	DATA P_OM_idName /'O3', 'NO2', 'NO', 'PAN', 'HNO3', 'SO2', 'CO', 'CH4', 'C2H6', 'NC4H10', 'C2H4', 'C3H6', 'OXYL', 'C5H8', 'APINEN', &
	                  'HCHO', 'CH3CHO', 'GLYOX', 'MGLYOX', 'CH3COE', '21', '22', '23', '24', '25', '26', '27', '28', '29', '30'/
		
	
	!>>> END <<<
	
	INTEGER :: P_unit

	CONTAINS
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE convert_specie_mech(spcName, factor, mech, ppbm, ret)

	
		CHARACTER(LEN=*), INTENT(INOUT) :: spcName
		REAL, INTENT(INOUT)             :: factor
		LOGICAL, INTENT(IN)             :: ppbm
		INTEGER, INTENT(OUT)            :: ret
		CHARACTER(LEN=*), INTENT(IN)    :: mech

!			IF(ppbm) THEN
				factor = factor/P_airWeight          
!			ELSE
!				factor = 1.
!			END IF		
		
				
			SELECT CASE(mech)
			CASE('CB07')
				CALL  convert_species_CB07(spcName, factor)
				ret = 0
			CASE('RACM')
				CALL  convert_species_RACM(spcName, factor)
				ret = 0
			CASE('RELACS')
				CALL convert_species_RELACS(spcName, factor)
                                ret = 0
			CASE DEFAULT 
				ret = -1
				RETURN
			END SELECT		
		

	END SUBROUTINE convert_specie_mech

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE convert_species_RACM(spcName, factor)

		CHARACTER(LEN=*), INTENT(INOUT) :: spcName
		REAL, INTENT(INOUT)             :: factor

 			SELECT CASE(TRIM(spcName))
        		CASE('ETH')
                		spcName = 'C2H6'
        		CASE('ETE')
                		spcName = 'C2H4'
        		CASE('HC3', 'HC5','HC8')
                		spcName = 'NC4H10'
                		factor   = factor/7.45
        		CASE('OLT', 'OLI')
                		spcName = 'C3H6'
                		factor   = factor/3.56
        		CASE('TOL', 'XYL', 'CSL')
               			spcName = 'OXYL'
                		factor   = factor/3.75
        		CASE('ISO', 'DIEN')
                		spcName = 'C5H8'
                		factor   = factor/1.66
        		CASE('API', 'LIM')
                		spcName = 'APINEN'
                		factor   = factor/4.19
        		CASE('ALD', 'DCB', 'MACR')
                		spcName = 'CH3CHO'
                		factor   = factor/6.35
        		CASE('GLY')
                		spcName = 'GLYOX'
        		CASE('MGLY')
                		spcName = 'MGLYOX'
        		CASE('KET', 'HKET')
                		spcName = 'CH3COE'
                		factor   = factor/3.12
 			END SELECT

	END SUBROUTINE convert_species_RACM

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

        SUBROUTINE convert_species_RELACS(spcName, factor)

                CHARACTER(LEN=*), INTENT(INOUT) :: spcName
                REAL, INTENT(INOUT)             :: factor

                        SELECT CASE(TRIM(spcName))
                        CASE('ETH')
                                spcName = 'C2H6'
                        CASE('ALKA')
                                spcName = 'NC4H10'
                                factor  = factor*(0.77+1.23+1.58)/7.45
                        CASE('ALKE')
                                spcName = 'C3H6'
                                factor  = factor*(1.04+1.04)/3.56
                        CASE('ARO')
                                spcName = 'OXYL'
                                factor  = factor*(0.87+1.04+1.04)/3.75
                        CASE('BIO')
                                spcName = 'C5H8'
                                factor  = factor/1.66
                        CASE('ALD')
                                spcName = 'CH3CHO'
                                factor  = factor/6.35
                        CASE('CRBO')
                                spcName = 'MGLYOX'
                        CASE('KET')
                                spcName = 'CH3COE'
                                factor  = factor/3.12
                        END SELECT

        END SUBROUTINE convert_species_RELACS


!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE convert_species_CB07(spcName, factor)

 		CHARACTER(*), INTENT(INOUT) :: spcName
		REAL, INTENT(INOUT)                   :: factor 

 			SELECT CASE(TRIM(spcName))
        		CASE('ETH')
                		spcName = 'C2H4'
        		CASE('FORM')
                		spcName = 'HCHO'
        		CASE('PAR')
                		spcName = 'NC4H10'
                		factor   = factor/3.0
        		CASE('OLE')
                		spcName = 'C3H6'
        		CASE('TOL', 'XYL', 'CRES')
                		spcName = 'OXYL'
                		factor   = factor/3.75
        		CASE('ISOP')
                		spcName = 'C5H8'
                		factor   = factor/1.66
        		CASE('ALD2', 'ISPD')
                		spcName = 'CH3CHO'
                		factor   = factor/6.35
        		CASE('MGLY')
                		spcName = 'MGLYOX'
 		END SELECT

	END SUBROUTINE convert_species_CB07

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

END MODULE rulesMOD
